unit clLookup;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, objcontinent, fgl, dialogs;

type

TLookupItem = class
  id    : integer;
  value : string;
end;


TLookupItemList = specialize TFPGObjectlist<TLookupItem>;
TLookupList     = specialize TFPGMap<string,TLookupItemList>;


{ TLookup }

TLookup = class
  fLookupList : TLookupList;
private
  function locate(const aFieldname: string; var aList: TLookupItemList): boolean;
public
  constructor create;
  destructor  destroy; override;

  procedure   Add(const aName : string; aLookupItem : TLookupItem);
  procedure   LoadContinent(alist : TContinentlist; aItems : TStrings);

  function    KetItemKey(const aMap, aValue : string) : integer;
  function    GetValueFromIndex(const aMap : string; const aIndex : integer) : string;
end;

implementation

{ TLookup }

constructor TLookup.create;
begin
  fLookupList := TLookupList.create
end;

destructor TLookup.destroy;
begin
  fLookupList.free
end;

procedure TLookup.Add(const aName: string; aLookupItem: TLookupItem);
var fLookupItemList : TLookupItemList;
begin
  if not fLookuplist.TryGetData(aName,fLookupItemList) then
    fLookupItemList := TLookupItemList.create;
  fLookupItemList.Add(aLookupItem);
  fLookuplist.AddOrSetData(aName,fLookupItemList);
end;

procedure TLookup.LoadContinent(alist : TContinentlist; aItems : TStrings);
var Item      : TLookupItem;
    Continent : TContinent;
begin
  for continent in aList do
  begin
    Item       := TLookupItem.Create;
    Item.id    := continent.ID;
    Item.value := continent.name;
    Add('continent',Item);
    aItems.Add(continent.name);
  end;
end;

function TLookup.KetItemKey(const aMap, aValue: string): integer;
var List  : TLookupItemList;
    Item  : TLookupItem;
begin
  result := -1;
  List := nil;
  if locate(aMap,List) then
  begin
    for Item in list do
    begin
      if Item.value = aValue then
      begin
        result := Item.id;
        break;
      end;
    end;
  end;
end;

function TLookup.GetValueFromIndex(const aMap: string;
                                   const aIndex: integer): string;
var MyList  : TLookupItemList;
    Item    : TLookupItem;
begin
  result := '';
  MyList := nil;
  if locate(aMap,MyList) then
  begin
    for Item in Mylist do
    begin
      if Item.id = aIndex then
      begin
        result := Item.value;
        break;
      end;
    end;
  end;
end;

function TLookup.locate(const aFieldname : string; var aList  : TLookupItemList) : boolean;
begin
  result := fLookupList.TryGetData(aFieldname,aList);
end;

end.

