unit cldatabase;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, sqlDB, dConnection, sqlite3conn, DB;

type

 { TMyDatabase }

 TMyDatabase = class
  fSQLQuery       : TSQLQuery;
  fSQLTransaction : TSQLTransaction;
  private
    fdberror: boolean;
    fRecordCount: integer;
  public
    constructor create;
    destructor  destroy; override;

    procedure GetTable(const aQuery : string);
    procedure TableCommit;

    function GetLastID: int64;
    function dataset : tdataset;

    property recordcount : integer read fRecordCount write fRecordCount;
    property dberror     : boolean read fdberror write fdberror;
end;

implementation

{ TDatabase }

constructor TMyDatabase.create;
begin
  fSQLQuery       := TSQLQuery.Create(nil);
  if DMConnection.SQLConnector1.ConnectorType <> 'SQLite3' then
  begin
    fSQLTransaction := TSQLTransaction.Create(fSQLQuery);
    fSQLTransaction.DataBase := DMConnection.SQLConnector1;
  end else
    fSQLQuery.Transaction    := DMConnection.SQLTransaction1;
  fSQLQuery.DataBase       := DMConnection.SQLConnector1;
end;

destructor TMyDatabase.destroy;
begin
  fSQLQuery.free;
  inherited destroy;
end;

procedure TMyDatabase.GetTable(const aQuery: string);
var FieldID : TField;
begin
  if fSQLQuery <> nil then
  begin
    fSQLQuery.SQL.Text := aQuery;
    fSQLQuery.Active   := true;
    FieldID := fSQLQuery.FieldByName('id');
    if assigned(FieldID) then
      FieldID.required := false;
  end;
end;

function TMyDatabase.GetLastID : int64;
var MyQuery : TSQLQuery;
    MyTrans : TSQLTransaction;
begin
  MyQuery := TSQLQuery.Create(nil);
  MyTrans := TSQLTransaction.Create(MyQuery);
  result  := -1;
  try
    MyQuery.Transaction := MyTrans;
    MyQuery.DataBase    := DMConnection.SQLConnector1;
    MyTrans.DataBase    := DMConnection.SQLConnector1;
    MyQuery.SQL.Text    := 'SELECT last_insert_rowid()';
    MyQuery.Active      := true;
    result              := MyQuery.Fields[0].AsLargeInt;
  finally
    MyQuery.free;
  end;
end;

function TMyDatabase.dataset: tdataset;
begin
  result := fSQLQuery;
end;

procedure TMyDatabase.TableCommit;
begin
  if DMConnection.SQLConnector1.ConnectorType <> 'SQLite3' then
    fSQLTransaction.Commit
  else
    dmConnection.SQLTransaction1.Commit;
end;

end.

