unit clLookup;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, cldatabase, fgl;

type

TLookupItem = class
  id    : integer;
  value : string;
end;


TLookupItemList = specialize TFPGObjectlist<TLookupItem>;
TLookupList     = specialize TFPGMap<string,TLookupItemList>;


{ TLookup }

TLookup = class(TMyDatabase)
  fLookupList : TLookupList;
private
  procedure GetItems(const aItems: TStrings);

  function locate(const aFieldname: string;
                  var aList: TLookupItemList): boolean;
public
  constructor create;
  destructor  destroy; override;

  procedure   Add(const aName : string; aLookupItem : TLookupItem);
  procedure   LoadContinent(aItems : TStrings);

  function    KetItemKey(const aMap, aValue : string) : integer;
  function    GetValueFromIndex(const aMap : string; const aIndex : integer) : string;
end;

implementation

{ TLookup }

constructor TLookup.create;
begin
  fLookupList := TLookupList.create;
  inherited create;
end;

destructor TLookup.destroy;
begin
  fLookupList.free;
  inherited destroy;
end;

procedure TLookup.Add(const aName: string; aLookupItem: TLookupItem);
var fLookupItemList : TLookupItemList;
begin
  if not fLookuplist.TryGetData(aName,fLookupItemList) then
    fLookupItemList := TLookupItemList.create;
  fLookupItemList.Add(aLookupItem);
  fLookuplist.AddOrSetData(aName,fLookupItemList);
end;

procedure TLookup.LoadContinent(aItems : TStrings);
begin
  try
    GetTable('SELECT id, continent FROM continents');
    GetItems(aItems);
    TableCommit;
  finally
    fSQLQuery.Active  := false;
  end;
end;

function TLookup.KetItemKey(const aMap, aValue: string): integer;
var MyList  : TLookupItemList;
    Item    : TLookupItem;
begin
  result := -1;
  MyList := nil;
  if locate(aMap,MyList) then
  begin
    for Item in Mylist do
    begin
      if Item.value = aValue then
      begin
        result := Item.id;
        break;
      end;
    end;
  end;
end;

function TLookup.GetValueFromIndex(const aMap: string;
                                   const aIndex: integer): string;
var MyList  : TLookupItemList;
    Item    : TLookupItem;
begin
  result := '';
  MyList := nil;
  if locate(aMap,MyList) then
  begin
    for Item in Mylist do
    begin
      if Item.id = aIndex then
      begin
        result := Item.value;
        break;
      end;
    end;
  end;
end;

procedure TLookup.GetItems(const aItems: TStrings);
var
  Item: TLookupItem;
begin
  while not fSQLquery.eof do
  begin
    Item       := TLookupItem.Create;
    Item.id    := fSQLquery.Fields[0].AsInteger;
    Item.value := fSQLquery.Fields[1].AsString;
    Add('continent', Item);
    aItems.Add(Item.value);
    fSQLquery.Next;
  end;
end;

function TLookup.locate(const aFieldname : string; var aList  : TLookupItemList) : boolean;
begin
  result := fLookupList.TryGetData(aFieldname,aList);
end;

end.

