unit clmain;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, stdctrls, clmainmediator, objWorld, clLookup,
  cldbworld;

type

{ TMain }

 TMain = class
  strict private
    fMediator  : TMainMediator;
    fNewRecord : boolean;
    fWorld     : TWorld;
    fLookup    : TLookup;
    fdatabase  : TDBWorld;
    function GetID: integer;
    function GetlastID: integer;
  public
    constructor create;
    destructor  destroy;  override;

    procedure init(aContinent : TStrings);
    procedure Save;
    procedure recordprior;
    procedure recordnext;
    procedure addrecord;
    procedure DeleteRecord;
    procedure lookupcontinent(const aValue : string);
    procedure Display;

    property Mediator : TMainMediator read fMediator;
    property NewRecord : boolean read fNewRecord write fNewRecord;
    property ID        : integer read GetID;
    property lastID    : integer read GetlastID;
end;

implementation

{ TMain }

uses dialogs;

constructor TMain.create;
begin
  fMediator  := TMainMediator.create;
  fworld     := TWorld.create;
  fLookup    := TLookup.create;
  fdatabase  := TDBWorld.create;
end;

destructor TMain.destroy;
begin
  fLookup.free;
  fdatabase.free;
  fworld.free;
  fMediator.free;
  inherited destroy;
end;

procedure TMain.init(aContinent: TStrings);
begin
  fLookup.LoadContinent(aContinent);
  fdatabase.GetValues('countries', fWorld);
  if fWorld.Countrylist.Count > 0 then
    Display;
end;

procedure TMain.Display;
var Country: TCountry;
begin
  Country := fWorld.FindRow;
  //lookup
  Country.continent := fLookup.GetValueFromIndex('continent',Country.continent_id);
  fMediator.ReadToComp(Country);
end;

procedure TMain.Save;
var Country : TCountry;
begin
  if fNewRecord then
  begin
    fWorld.IncreaseID;
    Country := TCountry.Create;
  end else
    Country := fWorld.FindRow;
  fMediator.CmpToWrite(Country);
  Country.continent_id := fLookup.KetItemKey('continent',Country.continent);

  if fNewRecord then
  begin
    fWorld.Countrylist.Add(Country);
    fWorld.ID := fWorld.LastID;
    fDatabase.SaveToDatabase(true);
    fNewRecord := false
  end else
    fDatabase.SaveToDatabase(false);
  if fdatabase.dberror then
    showmessage('record nod saved')
end;

procedure TMain.recordprior;
begin
  if fWorld.ID > 0 then
  begin
    fWorld.ID := fWorld.ID - 1;
    Display
  end;
end;

procedure TMain.recordnext;
begin
  if fWorld.ID < fWorld.LastID then
  begin
    fWorld.ID := fWorld.ID + 1;
    Display
  end;
end;

procedure TMain.addrecord;
begin
  fMediator.Clear;
  fNewRecord := true;
end;

procedure TMain.DeleteRecord;
var country   : TCountry;
    emptylist : boolean;
begin
  country := fWorld.FindRow;
  fDatabase.RowDelete(country.id);
  if fdatabase.dberror then
    showmessage('record not deleted')
  else
  begin
    emptylist := fWorld.RowDelete;
    if not fWorld.Error then
    begin
      if emptylist then
      begin
        country := fWorld.FindRow;
        fMediator.ReadToComp(country);
      end else
        AddRecord;
    end else
      showmessage('cannot delete item')
  end;
end;

procedure TMain.lookupcontinent(const aValue: string);
begin
  fLookup.KetItemKey('continent',aValue)
end;

function TMain.GetID: integer;
begin
  result := fWorld.ID;
end;

function TMain.GetlastID: integer;
begin
  result := fWorld.lastID;
end;

end.

