unit cldatabase;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, sqlDB, dConnection, sqlite3conn, DB, fgl, typinfo; //added ;;

type

 //new object
 TobjFielddef = class
   fieldname  : string;
   varlength  : integer;
   datatype   : string;
 end;

 TObjFielddeflist = specialize TFPGObjectList<TobjFielddef>;

 { TMyDatabase }

 TMyDatabase = class
  fSQLQuery       : TSQLQuery;
  fSQLTransaction : TSQLTransaction;
  private
    fdberror: boolean;
    fRecordCount: integer;
    //new
    fObjFielddeflist : TObjFielddeflist;
  protected
     procedure LoadAttributes;
  public
    constructor create;
    destructor  destroy; override;

    procedure GetTable(const aQuery : string; const aParams : array of variant);
    procedure TableCommit;
    //new
    procedure RowDelete(const aID: integer; const aTable: string);

    function GetLastID: int64;
    function dataset : tdataset;

    property recordcount : integer read fRecordCount write fRecordCount;
    property dberror     : boolean read fdberror write fdberror;
    //new
    property ObjFielddeflist : TObjFielddeflist read fObjFielddeflist write fObjFielddeflist;

end;

implementation

{ TDatabase }

constructor TMyDatabase.create;
begin
  fObjFielddeflist := TObjFielddeflist.create;
  fSQLQuery        := TSQLQuery.Create(nil);
  if DMConnection.SQLConnector1.ConnectorType <> 'SQLite3' then
  begin
    fSQLTransaction := TSQLTransaction.Create(fSQLQuery);
    fSQLTransaction.DataBase := DMConnection.SQLConnector1;
  end else
    fSQLQuery.Transaction    := DMConnection.SQLTransaction1;
  fSQLQuery.DataBase       := DMConnection.SQLConnector1;
end;

destructor TMyDatabase.destroy;
begin
  fObjFielddeflist.free;
  fSQLQuery.free;
  inherited destroy;
end;

procedure TMyDatabase.LoadAttributes;  //new
var index : integer;
    Fielddef : TobjFielddef;
begin
  fSQLQuery.First;
  for index := 0 to fSQLQuery.Fields.Count -1 do
  begin
    Fielddef := TobjFielddef.Create;
    Fielddef.fieldname := fSQLQuery.Fields[index].FieldName;
    Fielddef.varlength := fSQLQuery.Fields[index].DataSize;
    Fielddef.datatype  := GetEnumName(TypeInfo(TFieldType), Ord(fSQLQuery.Fields[index].DataType));
    ObjFielddeflist.Add(Fielddef);
  end;
end;

procedure TMyDatabase.GetTable(const aQuery: string; const aParams : array of variant);
var FieldID : TField;
begin
  if fSQLQuery <> nil then
  begin
    fSQLQuery.SQL.Text := aQuery;
    if fSQLQuery.Params.Count > 0 then
    begin
      fSQLQuery.Params[0].AsInteger := aParams[0];
    end;
    fSQLQuery.Active   := true;
    FieldID := fSQLQuery.FieldByName('id');
    if assigned(FieldID) then
      FieldID.required := false;
  end;
end;

function TMyDatabase.GetLastID : int64;
var MyQuery : TSQLQuery;
    MyTrans : TSQLTransaction;
begin
  MyQuery := TSQLQuery.Create(nil);
  MyTrans := TSQLTransaction.Create(MyQuery);
  result  := -1;
  try
    MyQuery.Transaction := MyTrans;
    MyQuery.DataBase    := DMConnection.SQLConnector1;
    MyTrans.DataBase    := DMConnection.SQLConnector1;
    MyQuery.SQL.Text    := 'SELECT last_insert_rowid()';
    MyQuery.Active      := true;
    result              := MyQuery.Fields[0].AsLargeInt;
  finally
    MyQuery.free;
  end;
end;

function TMyDatabase.dataset: tdataset;
begin
  result := fSQLQuery;
end;

procedure TMyDatabase.TableCommit;
begin
  if DMConnection.SQLConnector1.ConnectorType <> 'SQLite3' then
    fSQLTransaction.Commit
  else
    dmConnection.SQLTransaction1.Commit;
end;

procedure TMyDatabase.RowDelete(const aID: integer; const aTable : string);
begin
  dberror := true;
  fSQLQuery.SQL.Text := format('DELETE FROM %s WHERE id = :id',[aTable]);
  fSQLQuery.Params[0].AsInteger := aID;
  fSQLQuery.ExecSQL;
  dberror := fSQLQuery.RowsAffected = -1;
  if DMConnection.SQLConnector1.ConnectorType <> 'SQLite3' then
    fSQLTransaction.Commit
  else
    dmConnection.SQLTransaction1.Commit;
end;

end.

