unit clMyBaseMediator;

interface

uses classes, fgl, sysutils, controls,  stdctrls;

type TMyDeclarations = (dcString, dcInteger, dcFloat, dcDate);

TMyCollection = class
  fDisplayComponent : TControl;
  fClassname        : string;
  fOldValue         : string;
  fDatatype         : TMyDeclarations;
  fReadOnly         : boolean;
end;

Tmydictionary = specialize TFPGMap<string,TMyCollection>;

{ TMyBaseMediator }

TMyBaseMediator = class
  private
    fcomponentList    : Tmydictionary;
    function locate(const aFieldname : string; var cln : TMycollection) : boolean;
  protected
    procedure mdread(const aFieldname: string; const aValue: variant);

    function mdwrite(const aFieldname: string): variant;
  public
    constructor Create;
    destructor  Destroy; override;

    procedure Clear;

    procedure AddComp(aCompname : TControl; const  aFieldname: string; const aAcces : boolean = false);
    procedure SetAttributes(const aFieldname, aDatatype : string; const aVarlength : integer);
end;

implementation

constructor TMyBaseMediator.Create;
begin
  fcomponentList    := Tmydictionary.create;
end;

destructor TMyBaseMediator.Destroy;
begin
  fcomponentList.Free;
  inherited;
end;


procedure TMyBaseMediator.mdread(const aFieldname: string; const aValue: variant);
var cln : TMycollection;
begin
  cln := nil;
  if locate(aFieldname,cln) then
  begin
    if cln.fDisplayComponent is Tedit then
    begin
      case cln.fDatatype of
        dcString  : TEdit(cln.fDisplayComponent).Text := aValue;
        dcInteger : TEdit(cln.fDisplayComponent).Text := IntToStr(aValue);
      end;
    end else if cln.fDisplayComponent is TCombobox then
    begin
      Tcombobox(cln.fDisplayComponent).Text := aValue
    end else if cln.fDisplayComponent is TCheckbox then
    begin
      TCheckbox(cln.fDisplayComponent).Checked := aValue
    end;
  end;
end;

function TMyBaseMediator.mdwrite(const aFieldname : string) :variant;
var cln : TMycollection;
begin
  cln := nil;
  if locate(aFieldname,cln) then
  begin
    if cln.fDisplayComponent is TEdit then
    begin
      case cln.fDatatype of
        dcString  : result := TEdit(cln.fDisplayComponent).Text;
        dcInteger : result := StrToIntDef(TEdit(cln.fDisplayComponent).Text,0);
      end;
    end else if cln.fDisplayComponent is Tcombobox then
    begin
      result := TCombobox(cln.fDisplayComponent).Text;
    end else if cln.fDisplayComponent is Tcheckbox then
    begin
      result := Tcheckbox(cln.fDisplayComponent).checked;
    end;
  end;
end;


procedure TMyBaseMediator.AddComp(aCompname : TControl; const  aFieldname: string;  const aAcces : boolean);
var cln : TMyCollection;
begin
  cln := TMyCollection.Create;
  cln.fDisplayComponent := aCompname;
  cln.fClassname := aCompName.ClassName;
  cln.fDatatype := dcString;
  fComponentlist.Add(aFieldname,cln);
end;

procedure TMyBaseMediator.SetAttributes(const aFieldname, aDatatype: string;
  const aVarlength: integer);
var cln : TMycollection;
begin
  cln := nil;
  if locate(aFieldname,cln) then
  begin
    if cln.fDisplayComponent is TEdit then
    begin
      TEdit(cln.fDisplayComponent).MaxLength := aVarLength;
      if aDatatype = 'ftInteger' then
      begin
        TEdit(cln.fDisplayComponent).NumbersOnly := true;
        if aDatatype = 'ftInteger' then
          cln.fDatatype := dcInteger;
      end;
    end;
  end;
end;

function TMyBaseMediator.locate(const aFieldname : string; var cln : TMycollection) : boolean;
begin
  result := fComponentlist.TryGetData(aFieldname,cln)
end;


procedure TMyBaseMediator.Clear;
var cln : TMycollection;
    index : integer;
begin
  for index := 0 to fcomponentList.Count - 1 do
  begin
     cln := fComponentlist.Data[index];
     if cln.fDisplayComponent is TEdit then
       TEdit(cln.fDisplayComponent).Text := ''
     else if cln.fDisplayComponent is Tcombobox then
       Tcombobox(cln.fDisplayComponent).Text := ''
     else if cln.fDisplayComponent is Tcheckbox then
       Tcheckbox(cln.fDisplayComponent).checked := false;
  end;
end;

end.
