unit dConnection;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, sqldblib, sqldb, sqlite3conn;

type

  { TDMConnection }

  TDMConnection = class(TDataModule)
    SQLConnector1: TSQLConnector;
    SQLDBLibraryLoader1: TSQLDBLibraryLoader;
    SQLite3Connection1: TSQLite3Connection;
    SQLTransaction1: TSQLTransaction;
    procedure DataModuleCreate(Sender: TObject);
    procedure DataModuleDestroy(Sender: TObject);
  private
    procedure CreateTables;

  public
    function DBConnection : TSQLConnector;
  end;

var
  DMConnection: TDMConnection;

implementation

{$R *.lfm}

{ TDMConnection }

procedure TDMConnection.DataModuleCreate(Sender: TObject);
var rootdir : string;
begin
 rootdir := extractfiledir(paramstr(0));
 SQLDBLibraryLoader1.LibraryName := rootdir + '\sqlite3.dll';
 SQLDBLibraryLoader1.Enabled := true;
 SQLConnector1.DatabaseName  := rootdir + '\world.db3';
 SQLConnector1.Connected := true;
 CreateTables;
end;

procedure TDMConnection.DataModuleDestroy(Sender: TObject);
begin
  SQLConnector1.Connected := false;
end;

procedure TDMConnection.CreateTables;
var MyTables : TStrings;
begin
  MyTables := TStringlist.create;
  try
    SQLConnector1.GetTableNames(MyTables);
    if MyTables.IndexOf('countries') = -1 then
    begin
      SQLConnector1.ExecuteDirect(
      'CREATE TABLE "countries" (' +
      '   "id" INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,' +
      '   "countryname" VARCHAR (30) NOT NULL,' +
      '   "capital" VARCHAR (40) NOT NULL,' +
      '   "population" VARCHAR (20) NOT NULL,' +
      '   "continent_id" INTEGER NOY NULL DEFAULT 0);');
      SQLConnector1.ExecuteDirect('CREATE UNIQUE INDEX "countries_id" ON "countries" ("id");');
      SQLTransaction1.commit;
    end;
    if MyTables.IndexOf('cities') = -1 then
    begin
      SQLConnector1.ExecuteDirect(
      'CREATE TABLE "cities" (' +
      '   "id" INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,' +
      '   "country_id" INTEGER NOT NULL,' +
      '   "cityname" VARCHAR (30) NOT NULL,' +
      '   "major" VARCHAR (30) NOT NULL,' +
      '   "capital" BOOLEAN NOT NULL DEFAULT FALSE,' +
      '   "sightseeing" VARCHAR (100), ' +
      '   "square" INTEGER NOT NULL DEFAULT 0,' +
      '   "poprange_id" INTEGER NOT NULL,' +
      '   "township_id" INTEGER NOY NULL DEFAULT 0);');
      SQLConnector1.ExecuteDirect('CREATE UNIQUE INDEX "city_id" ON "cities" ("id");');
      SQLTransaction1.commit;
    end;
    if MyTables.IndexOf('continents') = -1 then
    begin
      SQLConnector1.ExecuteDirect(
      'CREATE TABLE "continents" (' +
      '   "id" INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,' +
      '   "continent" VARCHAR (30) NOT NULL);');
      SQLTransaction1.commit;
      SQLConnector1.ExecuteDirect(
      'INSERT INTO continents (continent) VALUES ' +
      '("Asia"),("North America"),("South America"),("Autralia"),' +
      '("Europe"),("Africa"),("Antartica");');
      SQLTransaction1.commit;
    end;
    if MyTables.IndexOf('popranges') = -1 then
    begin
      SQLConnector1.ExecuteDirect(
      'CREATE TABLE "popranges" (' +
      '   "id" INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,' +
      '   "rangename" VARCHAR (30) NOT NULL);');
      SQLTransaction1.commit;
      SQLConnector1.ExecuteDirect(
      'INSERT INTO popranges (rangename) VALUES ' +
      '("1 - 1.000"),("1.001 - 10.000"),("10.001 - 25.000"),("25.001 - 50.000"),' +
      '("50.001 - 75.000"),("75.001 - 100.000"),("100.001 - 250.000"),("250.001 - 500.000"),' +
      '("500.001 - 1.000.000"),("1.000.001 - 2.500.000"),("2.500.001 - 5.000.000"),("5.000.000 and more");');
      SQLTransaction1.commit;
    end;
    if MyTables.IndexOf('townships') = -1 then
    begin
      SQLConnector1.ExecuteDirect(
      'CREATE TABLE "townships" (' +
      '   "id" INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,' +
      '   "townshipname" VARCHAR (30) NOT NULL);');
      SQLTransaction1.commit;
      SQLConnector1.ExecuteDirect(
      'INSERT INTO townships (townshipname) VALUES ' +
      '("Village"),("Town"),("City");');
      SQLTransaction1.commit;
    end;
  finally
    MyTables.free
  end;
end;

function TDMConnection.DBConnection: TSQLConnector;
begin
  result := SQLConnector1;
end;

end.

