unit cldbWorld;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, cldatabase, SQLdb, objWorld, sqlite3conn, dConnection;

type

{ TDBWorld }

 TDBWorld = class(TMyDatabase)
 strict private
   fworld  : TWorld;
   procedure ReadValuesToObject;
 public
   procedure GetValues(const aTablename : string; aWorld  : TWorld); overload;
   procedure GetValues(aQuery : TStrings; aWorld  : TWorld); overload;
   procedure SaveToDatabase(const aNewRecord : boolean);
end;

implementation

{ TDBWorld }

procedure TDBWorld.GetValues(const aTablename: string; aWorld  : TWorld);
begin
  GetTable(format('select * from %s',[aTablename]),[]);
  fWorld := aWorld;
  ReadValuesToObject;
end;

procedure TDBWorld.GetValues(aQuery: TStrings; aWorld  : TWorld);
begin
  GetTable(aQuery.Text,[]);
  fWorld := aWorld;
  ReadValuesToObject;
end;

procedure TDBWorld.SaveToDatabase(const aNewRecord: boolean);
var Country  : TCountry;
    obJID    : integer;
    MyQuery  : TStrings;
begin
  Myquery := TStringlist.create;
  dberror := true;
  try
    if aNewRecord then
    begin
      MyQuery.AddText('INSERT INTO countries (countryname, capital, population, continent_id)');
      MyQuery.AddText('VALUES (:name, :capital, :population, :continent_id)');
      if fWorld.Countrylist.Count = 0 then
        objID := 0
      else
        objID := fWorld.Countrylist.Count - 1;
    end else
    begin
      MyQuery.AddText('UPDATE countries SET');
      MyQuery.AddText('countryname = :name, capital = :capital,');
      MyQuery.AddText('population = :population, continent_id = :continent_id');
      MyQuery.AddText('WHERE id = :id');
      objID := fWorld.fID;
    end;
    Country := fWorld.Countrylist[objID];
    fSQLQuery.SQL.Text := MyQuery.Text;
    fSQLQuery.Params[0].AsString := Country.name;
    fSQLQuery.Params[1].AsString := Country.capital;
    fSQLQuery.Params[2].AsString := Country.population;
    fSQLQuery.Params[3].AsInteger := Country.continent_id;
    if not aNewRecord then
      fSQLQuery.Params[4].AsInteger := Country.id;
    fSQLQuery.ExecSQL;
    dberror := fSQLQuery.RowsAffected = -1;
    if DMConnection.SQLConnector1.ConnectorType <> 'SQLite3' then
      fSQLTransaction.Commit
    else
      dmConnection.SQLTransaction1.Commit;
    if aNewRecord then
      Country.id := GetLastID;
  finally
    MyQuery.free;
  end;
end;

procedure TDBWorld.ReadValuesToObject;
var Country  : TCountry;
begin
  if not dataset.Active then
    dataset.Active := true;
  RecordCount := dataset.recordcount;
  if RecordCount > 0 then
  begin
    fWorld.fID := 0;
    dataset.first;
    while not dataset.eof do
    begin
      Country              := TCountry.create;
      Country.id           := dataset.FieldByName('id').AsInteger;
      Country.name         := dataset.FieldByName('countryname').AsString;
      Country.capital      := dataset.FieldByName('capital').AsString;
      Country.population   := dataset.FieldByName('population').AsString;
      Country.continent_id := dataset.FieldByName('continent_id').AsInteger;
      fWorld.Countrylist.Add(Country);
      dataset.next;
    end;
  end else
    fWorld.fID := -1;
  //this peace of code must be after reading the records.
  //If there are no records the definitions can still read
  if dataset.active then
    LoadAttributes;

  fWorld.fLastID := fWorld.Countrylist.Count - 1;
  dataset.Active := false;
end;

end.

