unit clcity;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, clcitymediator, clLookup, objcity, cldatabase,  cldbcity;

type

{ TFcity }

TFcity = class
 strict private
   fCountry_id : integer;
   fMediator  : TCityMediator;
   fNewRecord : boolean;
   fCities    : TCities;
   fLookup    : TLookup;
   fdatabase  : TDBCity;
   function GetID: integer;
   function GetlastID: integer;
 public
   constructor create;
   destructor  destroy;  override;

   procedure init(aPopulation, aCBTownship : TStrings);
   procedure Save;
   procedure recordprior;
   procedure recordnext;
   procedure addrecord;
   procedure DeleteRecord;
   procedure Display;

   property Mediator : TCityMediator read fMediator;
   property NewRecord : boolean read fNewRecord write fNewRecord;
   property ID        : integer read GetID;
   property lastID    : integer read GetlastID;
   property country_id: integer read fCountry_id write fCountry_id;
end;

implementation

uses dialogs;

constructor TFcity.create;
begin
  fMediator  := TCityMediator.create;
  fCities    := TCities.create;
  fLookup    := TLookup.create;
  fdatabase  := TDBCity.create;
end;

destructor TFcity.destroy;
begin
  fLookup.free;
  fdatabase.free;
  fCities.free;
  fMediator.free;
  inherited destroy;
end;

procedure TFcity.init(aPopulation, aCBTownship : TStrings);
var MyQuery  : TStrings;
    Fielddef : TobjFielddef; //new
begin
  MyQuery := TStringlist.create;
  try
    fLookup.LoadPopulation(aPopulation);
    flookup.Loadtownship(aCBTownship);
    MyQuery.Text := 'SELECT * FROM cities WHERE country_id = :id';
    fdatabase.GetValues(Myquery, fCities,[fCountry_id]);
    if fdatabase.ObjFielddeflist.Count > 0 then
    begin
      for Fielddef in fdatabase.ObjFielddeflist do
      begin
        fMediator.SetAttributes(Fielddef.fieldname, Fielddef.datatype, Fielddef.varlength);
      end;
    end;
    if fCities.Citylist.Count > 0 then
      Display;
  finally
    MyQuery.free
  end;
end;

procedure TFcity.Display;
var City: TCity;
begin
  City := fCities.FindRow;
  //lookup
  City.population := fLookup.GetValueFromIndex('popranges',City.poprange_id);
  City.township   := fLookup.GetValueFromIndex('townships',City.township_id);

  fMediator.ReadToComp(City);
end;

procedure TFcity.Save;
var City : TCity;
begin
  if fNewRecord then
  begin
    fCities.IncreaseID;
    City := TCity.Create;
  end else
    City := fCities.FindRow;
  fMediator.CmpToWrite(City);
  City.township_id := fLookup.KetItemKey('townships',City.township);
  City.poprange_id := fLookup.KetItemKey('popranges',City.population);
  if fNewRecord then
  begin
    fcities.Citylist.Add(City);
    fcities.ID := fcities.LastID;
    fDatabase.SaveToDatabase(true, fCountry_id);
    fNewRecord := false
  end;// else
    fDatabase.SaveToDatabase(false, fCountry_id);
  if fdatabase.dberror then
    showmessage('record nod saved')
end;

procedure TFcity.recordprior;
begin
  if fcities.ID > 0 then
  begin
    fcities.ID := fcities.ID - 1;
    Display
  end;
end;

procedure TFcity.recordnext;
begin
  if fcities.ID < fcities.LastID then
  begin
    fcities.ID := fcities.ID + 1;
    Display
  end;
end;

procedure TFcity.addrecord;
begin
  fMediator.Clear;
  fNewRecord := true;
end;

procedure TFcity.DeleteRecord;
var City      : TCity;
    emptylist : boolean;
begin
  City := fcities.FindRow;
  fDatabase.RowDelete(City.id,'cities');
  if fdatabase.dberror then
    showmessage('record not deleted')
  else
  begin
    emptylist := fCities.RowDelete;
    if not fCities.Error then
    begin
      if emptylist then
      begin
        City := fCities.FindRow;
        fMediator.ReadToComp(City);
      end else
        AddRecord;
    end else
      showmessage('cannot delete item')
  end;
end;

function TFcity.GetID: integer;
begin
  result := fCities.ID;
end;

function TFcity.GetlastID: integer;
begin
  result := fCities.lastID;
end;

end.

end.

