unit cldbcity;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, cldatabase, SQLdb, objbase, objcity, sqlite3conn, dConnection,
  typinfo, DB;

type

{ TDBCity }

 TDBCity = class(TMyDatabase)
 strict private
   fCities  : TCities;
   procedure ReadValuesToObject;
 public
   procedure GetValues(const aTablename : string; aCities  : TCities; const aParams : array of variant); overload;
   procedure GetValues(aQuery : TStrings; aCities  : TCities; const aParams : array of variant); overload;
   procedure SaveToDatabase(const aNewRecord : boolean; const aCountryID : integer);
end;


implementation

{ TDBCity }

procedure TDBCity.GetValues(const aTablename: string; aCities: TCities; const aParams : array of variant);
begin
  GetTable(format('select * from %s',[aTablename]), aParams);
  fCities := aCities;
  ReadValuesToObject;
end;

procedure TDBCity.GetValues(aQuery: TStrings; aCities: TCities; const aParams : array of variant);
begin
  GetTable(aQuery.Text, aParams);
  fCities := aCities;
  ReadValuesToObject;
end;

procedure TDBCity.ReadValuesToObject;
var City  : TCity;
    Fielddef : TobjFielddef;
begin
  if not dataset.Active then
    dataset.Active := true;
  RecordCount := dataset.recordcount;
  if RecordCount > 0 then
  begin
    fCities.fID := 0;
    dataset.first;
    while not dataset.eof do
    begin
      City              := TCity.create;
      City.id           := dataset.FieldByName('id').AsInteger;
      City.cityname     := dataset.FieldByName('cityname').AsString;
      City.capital      := dataset.FieldByName('capital').Asboolean;
      City.sightseeing  := dataset.FieldByName('sightseeing').AsString;
      City.square       := dataset.FieldByName('square').AsInteger;
      City.poprange_id  := dataset.FieldByName('poprange_id').AsInteger;
      City.township_id  := dataset.FieldByName('township_id').AsInteger;
      fCities.Citylist.Add(City);
      dataset.next;
    end;
  end else
    fCities.fID := -1;
  //this peace of code must be after reading the records.
  //If there are no records the definitions can still read
  if dataset.active then
    LoadAttributes;

  fCities.fLastID := fCities.Citylist.Count - 1;
  dataset.Active := false;
end;

procedure TDBCity.SaveToDatabase(const aNewRecord: boolean; const aCountryID : integer);
var City     : TCity;
    obJID    : integer;
    MyQuery  : TStrings;
begin
  Myquery := TStringlist.create;
  dberror := true;
  try
    if aNewRecord then
    begin
      MyQuery.AddText('INSERT INTO cities (cityname, capital, major, sightseeing,');
      MyQuery.AddText('square, poprange_id, township_id, country_id)');
      MyQuery.AddText('VALUES');
      MyQuery.AddText('(:cityname, :capital, :major, :sightseeing,');
      MyQuery.AddText(':square, :poprange_id, :township_id, :country_id)');
      if fCities.Citylist.Count = 0 then
        objID := 0
      else
        objID := fCities.Citylist.Count - 1;
    end else
    begin
      MyQuery.AddText('UPDATE cities SET');
      MyQuery.AddText('cityname = :cityname, capital = :capital, major = :major ,sightseeing = :sightseeing,');
      MyQuery.AddText('square = :square, poprange_id = :poprange_id, township_id = :township_id');
      MyQuery.AddText('WHERE id = :id AND country_id = :country_id');
      objID := fCities.fID;
    end;
    City := fCities.Citylist[objID];
    fSQLQuery.SQL.Text := MyQuery.Text;
    fSQLQuery.Params[0].AsString := City.cityname;
    fSQLQuery.Params[1].AsBoolean:= City.capital;
    fSQLQuery.Params[2].AsString := City.major;
    fSQLQuery.Params[3].AsString := City.sightseeing;
    fSQLQuery.Params[4].AsInteger := City.square;
    fSQLQuery.Params[5].AsInteger := City.poprange_id;
    fSQLQuery.Params[6].AsInteger := City.township_id;
    if aNewRecord then
      fSQLQuery.Params[7].AsInteger := aCountryID
    else
    begin
      fSQLQuery.Params[7].AsInteger := City.id;
      fSQLQuery.Params[8].AsInteger := aCountryID;

    end;
    fSQLQuery.ExecSQL;
    dberror := fSQLQuery.RowsAffected = -1;
    if DMConnection.SQLConnector1.ConnectorType <> 'SQLite3' then
      fSQLTransaction.Commit
    else
      dmConnection.SQLTransaction1.Commit;

    if aNewRecord then
      City.id := GetLastID;
  finally
    MyQuery.free;
  end;
end;

end.

