unit objWorld;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, fgl, objbase;

type
  TCountry = class
    id           : integer;
    name         : string;
    capital      : string;
    population   : string;
    continent_id : integer; //for database purpose
    continent    : string;
  end;

TCountrylist = specialize TFPGObjectList<TCountry>;

  TCityDetail = class
    country_id : integer; //for database purpose only
    cityname   : string;
    population : string;
    townrange  : string;
  end;

  TCityDetaillist = specialize TFPGObjectList<TCityDetail>;

{ TWorld }

TWorld = class(TObjectBase)
  fcountrylist    : TCountrylist;
  fCityDetaillist : TCityDetaillist;
private
  procedure Reindex;
  public
    constructor create;
    destructor  destroy; override;

    function FindRow : TCountry;
    function RowDelete  : boolean;

    procedure IncreaseID;

    property Countrylist    : Tcountrylist read fCountrylist;
    property CityDetaillist : TCityDetaillist read fCityDetaillist;
end;


implementation

{ TWorld }

constructor TWorld.create;
begin
  fCountrylist    := TCountrylist.create;
  fCityDetaillist := TCityDetaillist.create;
end;

destructor TWorld.destroy;
begin
  fCityDetaillist.free;
  fCountrylist.free;
  inherited destroy;
end;

procedure TWorld.IncreaseID;
begin
  fLastID := FLastID + 1;
  fID     := fLastID;
end;

procedure TWorld.Reindex;
var country : TCountry;
    index   : integer;
begin
  for index := 0 to fcountrylist.Count - 1 do
  begin
    country := fcountrylist[index];
    country.id := index;
    flastid := index;
  end;
end;

function TWorld.FindRow: TCountry;
begin
  try
    if assigned(fcountrylist) then
    begin
      if (fcountrylist.Count > 0) and (fID >= 0) then
        result := fCountrylist[fID]
    end else
      result := nil;
    except
      result := nil;
  end;
end;

function TWorld.RowDelete: boolean;
var myID : integer;
begin
  try
    fError := false;
    myID := fID;
    fcountrylist.Delete(fID);
    fcountrylist.Pack;
    if fcountrylist.Count = 0 then
    begin
      result := false;
      fID := -1;
    end  else
    begin
      if myID = fLastID then
      begin
        myID := myID - 1;
        if myID = -1 then
        begin
          fID := -1;
          result := false;
        end else
        begin
          fID := fID - 1;
          result := true;
        end;
      end else if myID = 0 then
      begin
        fID := -1;
        result := false;
      end else if myID < fLastID then
      begin
        result := true;
      end;
    end;
    Reindex;
  except
    fError := true
  end;
end;

end.

